
module Main (main) where

import Test.HUnit
import Text.Regex (mkRegex, matchRegex)
import RE (match)

main :: IO ()
main = do
          putStrLn "Us (laziness):"
          runTestTT (lazy_tests match)
          putStrLn "Us (nul):"
          runTestTT (nul_tests match)
          putStrLn "Us:"
          runTestTT (shared_tests match)
          putStrLn "Them:"
          runTestTT (shared_tests (matchRegex . mkRegex))
          putStrLn "All done!"
          return ()

mk_htest :: (String -> String -> Maybe [String])
         -> String -> (String, Maybe [String]) -> Test
mk_htest impl re (inp, res) = TestLabel ("inp:" ++ inp)
                            $ TestCase $ assertEqual inp res (impl re inp)

mk_testlist :: (String -> String -> Maybe [String])
            -> String -> [(String, Maybe [String])] -> Test
mk_testlist impl re irs = TestLabel ("re:" ++ re)
                        $ TestList
                        $ map (mk_htest impl re) irs

shared_tests :: (String -> String -> Maybe [String]) -> Test
shared_tests impl = TestList
                  $ map (uncurry (mk_testlist impl)) shared_tests_data

shared_tests_data :: [(String, [(String, Maybe [String])])]
shared_tests_data =
                [("", [
                       ("", Just []),
                       ("a", Just []),
                       ("abc", Just [])
                      ]),
                 ("^", [
                       ("", Just []),
                       ("a", Just []),
                       ("abc", Just [])
                      ]),
                 ("$", [
                       ("", Just []),
                       ("a", Just []),
                       ("abc", Just [])
                      ]),
                 ("^$", [
                       ("", Just []),
                       ("a", Nothing),
                       ("abc", Nothing)
                      ]),
                 ("a", [
                       ("", Nothing),
                       ("a", Just []),
                       ("b", Nothing),
                       ("abc", Just []),
                       ("bac", Just []),
                       ("bca", Just [])
                      ]),
                 ("^a", [
                       ("", Nothing),
                       ("a", Just []),
                       ("b", Nothing),
                       ("abc", Just []),
                       ("bac", Nothing),
                       ("bca", Nothing)
                      ]),
                 ("a^b", [
                       ("", Nothing),
                       ("a", Nothing),
                       ("b", Nothing),
                       ("ab", Nothing),
                       ("a^b", Nothing),
                       ("abc", Nothing),
                       ("dab", Nothing),
                       ("dabc", Nothing)
                      ]),
                 ("a$", [
                       ("", Nothing),
                       ("a", Just []),
                       ("b", Nothing),
                       ("abc", Nothing),
                       ("bac", Nothing),
                       ("bca", Just [])
                      ]),
                 ("a$b", [
                       ("", Nothing),
                       ("a", Nothing),
                       ("b", Nothing),
                       ("ab", Nothing),
                       ("a$b", Nothing),
                       ("abc", Nothing),
                       ("dab", Nothing),
                       ("dabc", Nothing)
                      ]),
                 ("^[a]$", [
                       ("", Nothing),
                       ("a", Just []),
                       ("b", Nothing),
                       ("aa", Nothing)
                      ]),
                 ("^[ab]$", [
                       ("", Nothing),
                       ("a", Just []),
                       ("b", Just []),
                       ("c", Nothing),
                       ("ab", Nothing)
                      ]),
                 ("^[^a]$", [
                       ("", Nothing),
                       ("a", Nothing),
                       ("b", Just []),
                       ("aa", Nothing)
                      ]),
                 ("^[^ab]$", [
                       ("", Nothing),
                       ("a", Nothing),
                       ("b", Nothing),
                       ("c", Just []),
                       ("ab", Nothing)
                      ]),
                 ("^[[:alnum:]]$", [
                       ("", Nothing),
                       ("1", Just []),
                       ("a", Just []),
                       ("A", Just []),
                       (" ", Nothing)
                      ]),
                 ("^[[:alpha:]]$", [
                       ("", Nothing),
                       ("1", Nothing),
                       ("a", Just []),
                       ("A", Just [])
                      ]),
                 ("^[^34[:alpha:]5]$", [
                       ("", Nothing),
                       ("1", Just []),
                       ("3", Nothing),
                       ("4", Nothing),
                       ("5", Nothing),
                       ("6", Just []),
                       ("a", Nothing),
                       ("A", Nothing)
                      ]),
                 ("^[[:blank:]]$", [
                       ("", Nothing),
                       (" ", Just []),
                       ("\t", Just []),
                       ("a", Nothing)
                      ]),
                 ("^[[:cntrl:]]$", [
                       ("", Nothing),
                       ("\003", Just []),
                       ("a", Nothing)
                      ]),
                 ("^[[:digit:]]$", [
                       ("", Nothing),
                       ("1", Just []),
                       ("12", Nothing),
                       ("a", Nothing)
                      ]),
                 ("^[[:graph:]]$", [
                       ("", Nothing),
                       (" ", Nothing),
                       ("\t", Nothing),
                       (",", Just []),
                       ("1", Just []),
                       ("a", Just []),
                       ("\003", Nothing)
                      ]),
                 ("^[[:lower:]]$", [
                       ("", Nothing),
                       ("1", Nothing),
                       ("a", Just []),
                       ("A", Nothing)
                      ]),
                 ("^[[:print:]]$", [
                       ("", Nothing),
                       (" ", Just []),
                       ("\t", Nothing),
                       (",", Just []),
                       ("1", Just []),
                       ("a", Just []),
                       ("\003", Nothing)
                      ]),
                 ("^[[:punct:]]$", [
                       ("", Nothing),
                       (" ", Nothing),
                       ("\t", Nothing),
                       (",", Just []),
                       ("1", Nothing),
                       ("a", Nothing),
                       ("\003", Nothing)
                      ]),
                 ("^[[:space:]]$", [
                       ("", Nothing),
                       (" ", Just []),
                       ("\t", Just []),
                       ("a", Nothing)
                      ]),
                 ("^[[:upper:]]$", [
                       ("", Nothing),
                       ("1", Nothing),
                       ("a", Nothing),
                       ("A", Just [])
                      ]),
                 ("^[[:xdigit:]]$", [
                       ("", Nothing),
                       ("1", Just []),
                       ("12", Nothing),
                       ("a", Just []),
                       ("f", Just []),
                       ("g", Nothing),
                       ("z", Nothing),
                       ("A", Just []),
                       ("F", Just []),
                       ("G", Nothing),
                       ("Z", Nothing)
                      ]),
                 ("^\\\\$", [
                       ("", Nothing),
                       ("\\", Just []),
                       ("1", Nothing),
                       ("12", Nothing),
                       ("a", Nothing),
                       ("A", Nothing),
                       (".", Nothing),
                       ("\001", Nothing),
                       ("\000", Nothing)
                      ]),
                 ("^.$", [
                       ("", Nothing),
                       ("1", Just []),
                       ("12", Nothing),
                       ("a", Just []),
                       ("A", Just []),
                       (".", Just []),
                       ("\001", Just []),
                       ("\000", Nothing)
                      ]),
                 ("^[.]$", [
                       ("", Nothing),
                       ("1", Nothing),
                       ("12", Nothing),
                       ("a", Nothing),
                       ("A", Nothing),
                       (".", Just []),
                       ("\001", Nothing),
                       ("\000", Nothing)
                      ]),
                 ("^\\.$", [
                       ("", Nothing),
                       ("1", Nothing),
                       ("12", Nothing),
                       ("a", Nothing),
                       ("A", Nothing),
                       (".", Just []),
                       ("\001", Nothing),
                       ("\000", Nothing)
                      ]),
                 ("^a$", [
                       ("", Nothing),
                       ("a", Just []),
                       ("b", Nothing),
                       ("abc", Nothing),
                       ("bac", Nothing),
                       ("bca", Nothing)
                      ]),
                 ("^a|b$", [
                       ("", Nothing),
                       ("a", Just []),
                       ("b", Just []),
                       ("c", Nothing),
                       ("aa", Just []),
                       ("ab", Just []),
                       ("ac", Just []),
                       ("ba", Nothing),
                       ("bb", Just []),
                       ("bc", Nothing),
                       ("ca", Nothing),
                       ("cb", Just []),
                       ("cc", Nothing),
                       ("abc", Just []),
                       ("acb", Just []),
                       ("bac", Nothing),
                       ("bca", Nothing),
                       ("cab", Just []),
                       ("cba", Nothing)
                      ]),
                 ("^(a|b)$", [
                       ("", Nothing),
                       ("a", Just ["a"]),
                       ("b", Just ["b"]),
                       ("c", Nothing),
                       ("aa", Nothing),
                       ("ab", Nothing),
                       ("ac", Nothing),
                       ("ba", Nothing),
                       ("bb", Nothing),
                       ("bc", Nothing),
                       ("ca", Nothing),
                       ("cb", Nothing),
                       ("cc", Nothing),
                       ("abc", Nothing)
                      ]),
                 ("^(a)b$|^a(b)", [
                       ("", Nothing),
                       ("a", Nothing),
                       ("b", Nothing),
                       ("c", Nothing),
                       ("aa", Nothing),
                       ("ab", Just ["", "b"]),
                       ("abc", Just ["", "b"])
                      ]),
                 ("^((abc){3})$", [
                       ("", Nothing),
                       ("a", Nothing),
                       ("b", Nothing),
                       ("c", Nothing),
                       ("abc", Nothing),
                       ("abca", Nothing),
                       ("abcab", Nothing),
                       ("abcabc", Nothing),
                       ("abcabca", Nothing),
                       ("abcabcab", Nothing),
                       ("abcabcabc", Just ["abcabcabc", "abc"]),
                       ("abcabcabca", Nothing),
                       ("abcabcabcab", Nothing),
                       ("abcabcabcabc", Nothing),
                       ("abcabcabcabca", Nothing),
                       ("abcabcabcabcab", Nothing),
                       ("abcabcabcabcabc", Nothing),
                       ("abcabcabcabcabca", Nothing),
                       ("abcabcabcabcabcab", Nothing),
                       ("abcabcabcabcabcabc", Nothing)
                      ]),
                 ("^((abc){3,})$", [
                       ("", Nothing),
                       ("a", Nothing),
                       ("b", Nothing),
                       ("c", Nothing),
                       ("abc", Nothing),
                       ("abca", Nothing),
                       ("abcab", Nothing),
                       ("abcabc", Nothing),
                       ("abcabca", Nothing),
                       ("abcabcab", Nothing),
                       ("abcabcabc", Just ["abcabcabc", "abc"]),
                       ("abcabcabca", Nothing),
                       ("abcabcabcab", Nothing),
                       ("abcabcabcabc", Just ["abcabcabcabc", "abc"]),
                       ("abcabcabcabca", Nothing),
                       ("abcabcabcabcab", Nothing),
                       ("abcabcabcabcabc", Just ["abcabcabcabcabc", "abc"]),
                       ("abcabcabcabcabca", Nothing),
                       ("abcabcabcabcabcab", Nothing)
                      ]),
                 ("^((abc){3,5})$", [
                       ("", Nothing),
                       ("a", Nothing),
                       ("b", Nothing),
                       ("c", Nothing),
                       ("abc", Nothing),
                       ("abca", Nothing),
                       ("abcab", Nothing),
                       ("abcabc", Nothing),
                       ("abcabca", Nothing),
                       ("abcabcab", Nothing),
                       ("abcabcabc", Just ["abcabcabc", "abc"]),
                       ("abcabcabca", Nothing),
                       ("abcabcabcab", Nothing),
                       ("abcabcabcabc", Just ["abcabcabcabc", "abc"]),
                       ("abcabcabcabca", Nothing),
                       ("abcabcabcabcab", Nothing),
                       ("abcabcabcabcabc", Just ["abcabcabcabcabc", "abc"]),
                       ("abcabcabcabcabca", Nothing),
                       ("abcabcabcabcabcab", Nothing),
                       ("abcabcabcabcabcabc", Nothing),
                       ("abcabcabcabcabcabca", Nothing),
                       ("abcabcabcabcabcabcab", Nothing),
                       ("abcabcabcabcabcabcabc", Nothing)
                      ])
                ]

lazy_tests :: (String -> String -> Maybe [String]) -> Test
lazy_tests impl = TestList
                $ map (uncurry (mk_testlist impl)) lazy_tests_data

lazy_tests_data :: [(String, [(String, Maybe [String])])]
lazy_tests_data =
                [
                 ("", [
                       (error "Unneeded", Just [])
                      ]),
                 ("^", [
                       (error "Unneeded", Just [])
                      ]),
                 ("a", [
                       ("a" ++ error "Unneeded", Just []),
                       ("bca" ++ error "Unneeded", Just [])
                      ]),
                 ("^a", [
                       ("a" ++ error "Unneeded", Just []),
                       ("b" ++ error "Unneeded", Nothing)
                      ]),
                 ("^a|b$", [
                       ("a" ++ error "Unneeded", Just [])
                      ])
                ]

nul_tests :: (String -> String -> Maybe [String]) -> Test
nul_tests impl = TestList
               $ map (uncurry (mk_testlist impl)) nul_tests_data

nul_tests_data :: [(String, [(String, Maybe [String])])]
nul_tests_data =
                [
                 ("^(.|\000)$", [
                       ("", Nothing),
                       ("1", Just ["1"]),
                       ("12", Nothing),
                       ("a", Just ["a"]),
                       ("A", Just ["A"]),
                       (".", Just ["."]),
                       ("\001", Just ["\001"]),
                       ("\000", Just ["\000"])
                      ])
                ]

